/**
 * JimBot - Java IM Bot
 * Copyright (C) 2006-2009 JimBot project
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package ru.jimbot.modules.chat;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

import ru.jimbot.util.Log;

/**
 *   
 * @author Prolubnikov Dmitry
 */
public class ChatQueue implements Runnable {
private Thread th;
private ChatServer srv;
private ChatProps psp;
//   
private ConcurrentLinkedQueue <MsgElement> mq;
//     .    
//   
private ConcurrentHashMap<Integer, ConcurrentLinkedQueue<MsgElement>> mmq;
//   
public ConcurrentHashMap <String,UinElement> uq;
private int sleepAmount = 10000;
private int counter=0; // 
public int msgCounter=1; // 
private boolean isNewMsg=true;
    
/** Creates a new instance of ChatQueue */
public ChatQueue(ChatServer s) {
srv = s;
psp = ChatProps.getInstance(srv.getName());
sleepAmount = psp.getIntProperty("chat.pauseOut");
mq = new ConcurrentLinkedQueue<MsgElement>();
uq = new ConcurrentHashMap<String,UinElement>();
mmq = new ConcurrentHashMap<Integer, ConcurrentLinkedQueue<MsgElement>>();
}
    
/**
*    
*/
public int statUsers() {
return uq.size();
}
    
/**
*     
*/
public int statQueue() {
return mq.size();
}
    
/**
*    
*/
public int statSend() {
return msgCounter;
}
    
/**
*    
*/
public void addMsg(String m, String user, int room) {
Log.debug(":   "+m);
isNewMsg=true;
mq.add(new MsgElement(m, msgCounter++, user, room));
}
    
/**
*   
*/
public void addUser(String uin, String buin, int room) {
Log.debug(":   "+uin + ", " + buin);
uq.put(uin,new UinElement(uin,buin,0,room));
}
    
/**
*   
*/
public void changeUser(String uin, String buin){
UinElement u = uq.get(uin);
u.baseUin = buin;
uq.put(uin, u);
}
    
/**
*  
*/
public void changeUserRoom(String uin, int room){
UinElement u = uq.get(uin);
u.room = room;
uq.put(uin, u);
}
    
/**
*   
*/
public void delUser(String uin){
Log.debug(":    "+uin);
uq.remove(uin);
}
    
/**
*       
*/
public boolean testUser(String uin) {
return uq.containsKey(uin);
}
    
/**
*          
*/
public synchronized String createMsg(String uin) {
String s = "";
UinElement u = uq.get(uin);
if(!mmq.containsKey(u.room)) return "";
if(mmq.get(u.room).isEmpty()) return "";
for(MsgElement m:mmq.get(u.room)){
if(m.countID>u.lastMsg){
if(!m.userSN.equalsIgnoreCase(uin) || !psp.getBooleanProperty("chat.ignoreMyMessage"))
s += m.msg + '\n';
u.lastMsg = m.countID;
}
}
//  ,         
//((ChatCommandProc) srv.cmd).testState(uin);
if(uq.containsKey(uin)) uq.put(uin,u);
if(s.length()>1) return s.substring(0, s.length()-1);
return s;
}
    
public synchronized void send(){
try {
if (uq.size() == 0)
return;
if (mq.size() == 0)
return;
//      
while (!mq.isEmpty()) {
MsgElement m = mq.poll();
ConcurrentLinkedQueue<MsgElement> q;
if (!mmq.containsKey(m.room))
q = new ConcurrentLinkedQueue<MsgElement>();
else
q = mmq.get(m.room);
q.add(m);
mmq.put(m.room, q);
}
//  ,    ,  -
//   
for (String c : uq.keySet()) {
String s = createMsg(c);
((ChatCommandProc) srv.cmd).testState(c);
if (!s.equals("") && uq.containsKey(c))
srv.getIcqProcess(uq.get(c).baseUin).mq.add(uq.get(c).uin,
s);
}
//    (     
for (int room : mmq.keySet()) {
if (!mmq.get(room).isEmpty())
mmq.get(room).clear();
}
} catch (Exception ex) {
ex.printStackTrace();
}
}
        
public void start(){
th = new Thread(this);
th.setPriority(Thread.NORM_PRIORITY);
th.start();
}
    
public synchronized void stop() {
th = null;
notify();
}
    
public void run() {
Thread me = Thread.currentThread(); 
while (th == me) {
send();
try {
th.sleep(sleepAmount);
} catch (InterruptedException e) { break; }             
}
th=null;
}

/**
*   
* @author spec
*/
public class MsgElement {
public String msg="";
public int countID=0;
public String userSN=""; //     
public int room=0;
        
MsgElement(String s, int id, String user, int room) {
msg = s;
countID = id;
userSN = user;
this.room = room;
}
}
    
/**
*     
* @author spec
*/
public class UinElement {
public String uin="";
public String baseUin="";
public int lastMsg=0;
public int room = 0;
        
UinElement(String u, String bu, int lm, int room) {
uin = u;
baseUin = bu;      
lastMsg = lm;
this.room = room;
}
}
}
